// call the packages we need
require('express-async-errors');
let express       = require('express');
let oauth2Server  = require('oauth2-server');
let load          = require('express-load');
let MemoryStore   = require('./model');
let DynamicRoutes = require('./services/dynamicRoutes');
let responseService = require('./services/response');
const { NotFoundError, MethodNotAllowedError, UnauthorizedError } = require('~errors');

let app = express();
let ip = process.env.IP || 'localhost';
let port = process.env.PORT || 3000;
let address = ip + ':' + port;
let dynamicRoutes = new DynamicRoutes(app, address);

const isProduction = process.env.NODE_ENV === 'production';

// configure app to use bodyParser()
// this will let us get the data from a POST
app.use(express.urlencoded({ extended: true }));
app.use(express.json());

// configure oauth2 sever
app.oauth = new oauth2Server({
    model: new MemoryStore(),
});

// configure cross origin
app.use(function(req, res, next) {
    res.header("Access-Control-Allow-Origin", "*");
    res.header("Access-Control-Allow-Headers", "*");
    res.header("Access-Control-Allow-Methods", "*");
    next();
});

// load routes
load('./routes').into(app);
dynamicRoutes.createRoutes();

function loggerMiddleware(error, req, res, next) {
    console.log(error);
    next(error);
}

function catchAllMiddleware(error, req, res, next) {
    if (error instanceof NotFoundError) {
        return responseService.throwNotFound(res);
    }

    if (error instanceof MethodNotAllowedError) {
        return responseService.throwMethodNotAllowed(res);
    }

    if (error instanceof UnauthorizedError) {
        return responseService.throwUnauthorized(res);
    }

    let errorStack = error.stack;
    if (isProduction) {
        errorStack = {};
    }
    responseService.error(res, 500, 500, 'Internal Server Error', errorStack);
}

// Exception handling
app.use(loggerMiddleware);
app.use(catchAllMiddleware);

// start server
app.listen(port);
console.log('Server started on ' + address);
